"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = exports.shorthands = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const directory_fetcher_1 = require("@pnpm/directory-fetcher");
const fs_indexed_pkg_importer_1 = require("@pnpm/fs.indexed-pkg-importer");
const fs_is_empty_dir_or_nothing_1 = require("@pnpm/fs.is-empty-dir-or-nothing");
const plugin_commands_installation_1 = require("@pnpm/plugin-commands-installation");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const error_1 = require("@pnpm/error");
const rimraf_1 = __importDefault(require("@zkochan/rimraf"));
const render_help_1 = __importDefault(require("render-help"));
const deployHook_1 = require("./deployHook");
const logger_1 = require("@pnpm/logger");
const deployCatalogHook_1 = require("./deployCatalogHook");
exports.shorthands = plugin_commands_installation_1.install.shorthands;
function rcOptionsTypes() {
    return plugin_commands_installation_1.install.rcOptionsTypes();
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return plugin_commands_installation_1.install.cliOptionsTypes();
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['deploy'];
function help() {
    return (0, render_help_1.default)({
        description: 'Experimental! Deploy a package from a workspace',
        url: (0, cli_utils_1.docsUrl)('deploy'),
        usages: ['pnpm --filter=<deployed project name> deploy <target directory>'],
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: "Packages in `devDependencies` won't be installed",
                        name: '--prod',
                        shortAlias: '-P',
                    },
                    {
                        description: 'Only `devDependencies` are installed regardless of the `NODE_ENV`',
                        name: '--dev',
                        shortAlias: '-D',
                    },
                    {
                        description: '`optionalDependencies` are not installed',
                        name: '--no-optional',
                    },
                ],
            },
            common_cli_options_help_1.FILTERING,
        ],
    });
}
exports.help = help;
async function handler(opts, params) {
    if (!opts.workspaceDir) {
        throw new error_1.PnpmError('CANNOT_DEPLOY', 'A deploy is only possible from inside a workspace');
    }
    const selectedDirs = Object.keys(opts.selectedProjectsGraph ?? {});
    if (selectedDirs.length === 0) {
        throw new error_1.PnpmError('NOTHING_TO_DEPLOY', 'No project was selected for deployment');
    }
    if (selectedDirs.length > 1) {
        throw new error_1.PnpmError('CANNOT_DEPLOY_MANY', 'Cannot deploy more than 1 project');
    }
    if (params.length !== 1) {
        throw new error_1.PnpmError('INVALID_DEPLOY_TARGET', 'This command requires one parameter');
    }
    const deployedDir = selectedDirs[0];
    const deployDirParam = params[0];
    const deployDir = path_1.default.isAbsolute(deployDirParam) ? deployDirParam : path_1.default.join(opts.dir, deployDirParam);
    if (!(0, fs_is_empty_dir_or_nothing_1.isEmptyDirOrNothing)(deployDir)) {
        if (!opts.force) {
            throw new error_1.PnpmError('DEPLOY_DIR_NOT_EMPTY', `Deploy path ${deployDir} is not empty`);
        }
        logger_1.logger.warn({ message: 'using --force, deleting deploy path', prefix: deployDir });
    }
    await (0, rimraf_1.default)(deployDir);
    await fs_1.default.promises.mkdir(deployDir, { recursive: true });
    const includeOnlyPackageFiles = !opts.deployAllFiles;
    await copyProject(deployedDir, deployDir, { includeOnlyPackageFiles });
    await plugin_commands_installation_1.install.handler({
        ...opts,
        confirmModulesPurge: false,
        // Deploy doesn't work with dedupePeerDependents=true currently as for deploy
        // we need to select a single project for install, while dedupePeerDependents
        // doesn't work with filters right now.
        // Related issue: https://github.com/pnpm/pnpm/issues/6858
        dedupePeerDependents: false,
        depth: Infinity,
        hooks: {
            ...opts.hooks,
            readPackage: [
                ...(opts.hooks?.readPackage ?? []),
                deployHook_1.deployHook,
                deployCatalogHook_1.deployCatalogHook.bind(null, opts.catalogs ?? {}),
            ],
        },
        frozenLockfile: false,
        preferFrozenLockfile: false,
        saveLockfile: false,
        virtualStoreDir: path_1.default.join(deployDir, 'node_modules/.pnpm'),
        modulesDir: path_1.default.relative(deployedDir, path_1.default.join(deployDir, 'node_modules')),
        rawLocalConfig: {
            ...opts.rawLocalConfig,
            // This is a workaround to prevent frozen install in CI envs.
            'frozen-lockfile': false,
        },
        includeOnlyPackageFiles,
    });
}
exports.handler = handler;
async function copyProject(src, dest, opts) {
    const { filesIndex } = await (0, directory_fetcher_1.fetchFromDir)(src, opts);
    const importPkg = (0, fs_indexed_pkg_importer_1.createIndexedPkgImporter)('clone-or-copy');
    importPkg(dest, { filesMap: filesIndex, force: true, resolvedFrom: 'local-dir' });
}
//# sourceMappingURL=deploy.js.map